/* LIQUIDITY AND THE INTERNATIONAL ALLOCATION OF ECONOMIC ACTIVITY */
/* ANTONIO RODRIGUEZ-LOPEZ */
/* CREATING COMPUSTAT DATASET FOR ANALYSIS */
/* NAME OF FINAL FILE IS firmlevel.dta
which means that it includes NA and Globa data, excludes
finacial firms (SIC Codes 6000-6799), and has all nominal 
variables converted to USD */
/* Datadates start on 01/2011 and end on 05/2018, meaning
that I have full fiscal years 2011-2017 (06/2011-05/2018) */
/* XR data is up to April 2018, so that to be able to have
the full 2017 fiscal year I had to use the April 2018 xr data
for May 2018 --- this added 195 more observations for the
fiscal year 2017 */


clear all
set more off

*MAC OR PC
*If PC
*local s "\"
*local m "G:\My Drive\"
*If MAC
local s "/"
local m "`s'Data Supplement`s'"

*Directories 

local datadir "`m'data`s'Compustat`s'"
local xrdir "`m'data`s'Compustat_xr`s'"
local pcpidir "`m'data`s'pcpi`s'"
local cmedir "`m'data`s'cme`s'"
local outdir "`m'data`s'firmlevel`s'"


use "`datadir'Compustat_NAandGLOBAL_selectedvars.dta", clear


/* DATA FROM FISCAL YEAR 2011 FOR 47 COUNTRIES */

drop if fyear<2011

keep if loc=="ARE" |  loc=="ARG" |  loc=="AUS" |  loc=="AUT" |  loc=="BEL" |  loc=="BMU" |  loc=="BRA" |  loc=="CAN" |  loc=="CHE" |  loc=="CHL" |  loc=="CHN" |  loc=="COL" |  loc=="DEU" |  loc=="DNK" |  loc=="ESP" |  loc=="FIN" |  loc=="FRA" |  loc=="GBR" |  loc=="GRC" |  loc=="HKG" |  loc=="IDN" |  loc=="IND" |  loc=="IRL" |  loc=="ISR" |  loc=="ITA" |  loc=="JPN" |  loc=="KOR" |  loc=="LUX" |  loc=="MEX" |  loc=="MYS" |  loc=="NLD" |  loc=="NOR" |  loc=="NZL" |  loc=="PAK" |  loc=="PHL" |  loc=="POL" |  loc=="PRT" |  loc=="RUS" |  loc=="SAU" |  loc=="SGP" | loc=="SWE" |  loc=="THA" |  loc=="TUR" |  loc=="TWN" |  loc=="USA" |  loc=="VNM" |  loc=="ZAF" 

/* DROP FINANCIAL FIRMS */
destring sic, replace
drop if sic > 5999 & sic < 6800

/* FIX OUTLIERS IN RAW DATA */

replace curcd="VND" if fyear==2013 & gvkey=="286468"


/* MERGE XR DATA */
/* XR data is up to April 2018, but I copied April 2018 exchange rates in May 2018
so that I can use the entire 2017 fiscal year */
merge m:1 curcd datadate using "`xrdir'monthly_xr_USD.dta"
keep if _merge==3
drop _merge


/* -----------------------------------------------------------  */
/* CLEANING DUPLICATES */
/* DUPLICATE CLEANING IS BASED ON:
  https://robsonglasscock.wordpress.com/2018/04/12/gvkey-and-datadate-or-fyear-duplicates-in-compustat/ */

/* I USED THE FILE BELOW TO GET RID OF DUPLICATES IN GVKEY DATADATE
IT MOSTLY HAPPENS FROM 1) FIRMS ISSUING STATEMENTS IN BOTH FS AND INSTL FORMATS
AND 2) FROM INTERNATIONAL FIRMS ALSO ISSUING STATEMENTS IN BOTH THE NA
AND GLOBAL DATABASES. THE CLEANING MOSTLY KEEPS THE INFO FROM THE STATEMENTS
THEY RELEASED IN THE US, SO THEY ARE IN USD */

destring gvkey, replace
duplicates tag gvkey datadate, gen(dup)
tab dup
egen mc= rowmiss(curcd- exrusd12m)
bys gvkey datadate: egen minmc= min(mc)
gen del= 0
replace del=1 if dup != 0 & gvkey[_n]== gvkey[_n+1] & minmc != mc | ///
dup !=0 & gvkey[_n]== gvkey[_n-1] & minmc != mc
order gvkey datadate dup mc minmc del
tab indfmt if del==1
list gvkey sic if del==1 & indfmt=="INDL"
count if del==1 & indfmt=="INDL"
drop if del==1
duplicates tag gvkey datadate, gen(dup2)
tab dup2
gen tie=0
bys gvkey datadate: replace tie=1 if mc[_n]== mc[_n+1]
order gvkey datadate dup dup2 mc minmc del tie
drop if tie==1

/* Drop other problematic observations */

drop if gvkey==222639 /* drops 2 observations */
drop if indfmt=="FS" /*It drops 26 observations */

/* Up to here there are no duplicates with gvkey datadate
but duplicates remain for fyear because of some firms
changing their fiscal year end month */

duplicates tag gvkey fyear, gen(dup3)
drop minmc
bys gvkey fyear: egen minmc= min(mc)
order gvkey datadate fyear dup mc minmc del tie
replace del=1 if dup3==1 & gvkey[_n]== gvkey[_n+1] & minmc != mc | ///
dup3 ==1 & gvkey[_n]== gvkey[_n-1] & minmc != mc
drop if del==1
bys gvkey fyear: replace tie=1 if mc[_n]== mc[_n+1]
drop if tie==1

/* SECOND ROUND */
duplicates tag gvkey fyear, gen(dup4)
drop minmc
bys gvkey fyear: egen minmc= min(mc)
replace del=1 if dup4==1 & gvkey[_n]== gvkey[_n+1] & minmc != mc | dup4 ==1 & gvkey[_n]== gvkey[_n-1] & minmc != mc
drop if del==1
bys gvkey fyear: replace tie=1 if mc[_n]== mc[_n+1]
drop if tie==1

/* LAST ROUND */

replace del=1 if dup4==2 & gvkey[_n]== gvkey[_n+1] & minmc != mc | dup4 ==2 & gvkey[_n]== gvkey[_n-1] & minmc != mc
drop if del==1
duplicates tag gvkey fyear, gen(dup5)
tab dup5
drop dup dup2 dup3 dup4 dup5 mc minmc del tie


/* FINISH DUPLICATES CLEANING */
/* -----------------------------------------------------------  */



/* THIS CONVERTS EVERYTHING TO USD 
WE USE EXRUSD (END OF THE MONTH IN DATADATE) */

gen exr=exrusd 

/* CONVERT ALL NOMINAL VARIABLES TO USD */
foreach v in "sale" "cogs" "xsga" "dp" "oibdp" "ebit" "ebitda" "oiadp" "txt" "pi" "ib" "xint" "nopi" "spi" "teq" "seq" "ceq" "xrd" "dv" "dvc" "dvt" "ibc" "dpc" "capx" "ppegt" "ppent" "ni" "mkvalt" "bkvlps" "epsfi" "epsfx" "epspi" "epspx" "ceql" "dvp" "dvpsp_c" "dvpsx_c" "prcc_c" "prcc_f" "dvpsp_f" "dvpsx_f" "epsexcon" "epsexnc" "epsincon" "epsinnc" {
replace `v'=`v'/exr
}


compress
encode loc, gen(loc2)


/* CONVERT ALL NOMINAL VARIABLES TO REAL USD (PCEPI, 2012=100) */

gen month=mofd(datadate)
format month %tm
merge m:1 month using "`pcpidir'pcepi_monthly.dta"
keep if _merge==3
drop _merge

foreach v in "sale" "cogs" "xsga" "dp" "oibdp" "ebit" "ebitda" "oiadp" "txt" "pi" "ib" "xint" "nopi" "spi" "teq" "seq" "ceq" "xrd" "dv" "dvc" "dvt" "ibc" "dpc" "capx" "ppegt" "ppent" "ni" "mkvalt" "bkvlps" "epsfi" "epsfx" "epspi" "epspx" "ceql" "dvp" "dvpsp_c" "dvpsx_c" "prcc_c" "prcc_f" "dvpsp_f" "dvpsx_f" "epsexcon" "epsexnc" "epsincon" "epsinnc" {
replace `v'=`v'/pcepi
} 

tempfile compustat
save `compustat'

*Gain about 255 emp observations of employment
use "`datadir'Compustat_NAandGLOBAL_selectedvars.dta", clear
keep if emp!=.
destring gvkey, replace
collapse (first) empfirst=emp curcdfirst=curcd (last) emplast=emp curcdlast=curcd, by(gvkey fyear)
gen emp=empfirst if empfirst==emplast
tsset gvkey fyear
replace emp=empfirst if empfirst!=emplast & curcdfirst=="JPY"
replace emp=emplast if empfirst!=emplast & curcdfirst!="JPY" & curcdlast=="USD" & emplast!=0
replace emp=empfirst if empfirst!=emplast & curcdfirst!="JPY" & curcdlast=="USD" & emplast==0
replace emp=empfirst if empfirst!=emplast & gvkey==15581
replace emp=emplast if emp==.
keep gvkey fyear emp
ren emp emp_
tempfile emp
save `emp'

use `compustat', clear
merge 1:1 gvkey fyear using `emp'
drop if _merge==2
drop _merge
drop emp
ren emp_ emp


*Two-digit SIC
gen sic2=int(sic/100)

*Drop negative sales and negative seq observations
*And some cleaning of oibdp observations
drop if sale<0.0000001
drop if seq<0.0000001
replace oibdp=sale-cogs-xsga if oibdp==.
replace oibdp=sale-cogs-xsga if gvkey==226817
replace oibdp=. if gvkey==252476 & fyear==2016
replace oibdp=. if gvkey==221199 & fyear==2016


*Create new market value variable (will be very similar to one included, but will gain 5968 observations)
gen mkvalt2=prcc_c*csho
gen mkvalt3=prcc_f*csho
replace mkvalt3=mkvalt if mkvalt3==.
replace mkvalt3=mkvalt2 if mkvalt3==.
replace mkvalt3=. if mkvalt3==0
replace mkvalt2=. if mkvalt2==0
drop mkvalt mkvalt2
ren mkvalt3 mkvalt


/* ADD STOCK/BOND ACCEPTABILITY DATA FROM CME -- APRIL 2019 */

merge m:1 gvkey using "`cmedir'gvkey_CME_acceptability.dta"
drop _merge
replace bond_acc=0 if bond_acc==.
replace stock_acc=0 if stock_acc==.
replace accind=0 if accind==.

keep gvkey fyear bond_acc stock_acc accind sale oibdp emp mkvalt xrd seq sic loc2 sic sic2 loc
save "`outdir'firmlevel.dta", replace

